/****************************************************************/
/*								*/
/*								*/
/* (C) Copyright Motorola Inc, 1994.				*/
/*  All right reserved 						*/
/*                                                              */
/* Author: Uri Trichter                                         */
/*								*/
/* Description:							*/
/*     Simple Driver Set - TRASPARENT specific routines.	*/
/*								*/
/* Routines							*/
/*								*/
/*     trans_init						*/
/*     trans_tx						*/
/*     trans_rx_buffers					*/
/*     trans_tx_done						*/
/*     trans_interrupt						*/
/*								*/
/****************************************************************/

#include "setup.h"
#include "simple.h"
#include "imp.h"
#include "trans.h"

extern SCC_TABLE scc_table;
extern IMP     *imp;

void trans_interrupt(int scc_num);

/********************************************************
 * routine:	trans_init
 *
 * description:
 *     Initialize the SCC parameter ram and register as a TRASPARENT
 *     controller. 
 *
 * arguments:
 *     scc_num          - the number of the SCC channel (0-2)
 *     specific_info	- pointer to extra information structure 
 *
 * return code:
 *     None
 *
 ********************************************************/
void trans_init(	int scc_num , void *specific_info)
{
	struct bisync_pram *trans_pram;
	struct scc_regs      *regs;
	unsigned int div;       /* dividor */
	TRANS_SETUP	*set;

	/* scc_num must be valid and initialized. */
	if (scc_num < 0 || scc_num >= NUM_OF_SCCS )
		return;
    
	/* initialize the scc_table struct with the specific protocol interrupt */
	scc_table[scc_num].inter_ptr             = trans_interrupt;
	/*
	* INITIALIZE TRASPARENT PARAMATER RAM
	* ---------------------------------
	*/

	/* Pointer to corresponding trans parameter ram */
	trans_pram = (struct bisync_pram *)&imp->pram[scc_num].scc.pscc.b;

	trans_pram->rfc= 0;		/* Rx FC register */
	trans_pram->tfc= 0;		/* Tx FC register */
	trans_pram->rblen= scc_table[scc_num].max_frame_length;	/* Rx buffer length */
	
	trans_pram->rbuf = 0;		/* start from bd = 0 */
	trans_pram->tbuf = (short)(sizeof( IMP_BD)*NUM_OF_BDS);	/* start tbd's from first bd */

	/*
	* INITIALIZE SCC REGISTERS AREA
	* -----------------------------
	*/

	/* Set pointer to the special_pram which points to comm_settings */
	set = (TRANS_SETUP *)specific_info;

	/* Pointer to SCC registers area */
	regs = (struct scc_regs *)&(imp->scc_regs[scc_num]);

	/* Set scc conf registers to the baud rate*/
	div= (SYSTEM_CLOCK/set->baud); /* set it to 9600bps */

	if( div > MAX_DIV ){ 
		regs->conf = 0x0001;	/* set div4 bit */ 
		div >>=2;
	}
	else
		regs->conf = 0;
	div--;

	regs->conf |= (div <<1 );	/* SCC configuration register */

	if( set-> ex_rx_clk_source )
		regs->conf |= RCS;	/* reciver gets the external clock */

	if( set->ex_tx_clk_source )
		regs->conf|= TCS;	/* transmiter gets the external clock */

	regs->mode = 0;

	if( set->promiscuous)
 		regs->mode |= PROM_TRANS | TR_NTSYM;  /* totaly transparent mode */

 	regs->mode |=  set->diag_bits;
	
	if( set->reverse_data)
		regs->mode |= TR_REVD;

	if(set->externl_sync)
		regs->mode |= TR_EXSYN;

	/*
	* mask all the events
	*/
	regs->mask = TRANS_TX | TRANS_RX | TRANS_TXE | TRANS_BUSY; 
	
	/* set the sync register */
	regs->sync = set->sync;
	/*
	* enable SCC receive transmit operation
	*/
	regs->mode |= ENABLE_TX | ENABLE_RX;
}


/********************************************************
 * routine:	trans_tx
 *
 * description:
 *	Write the details of a given buffer to a transmit
 *	buffer descriptor of given trans channels.
 *
 * arguments:
 *	scc_num -	the number of the SCC channel (0-3)
 *	buf	-	pointer to the buffer to be transmitted
 *	length	-	buffer length
 *
 * return code:
 *	BUSY	- Indicate that no empty transmit buffer descriptor
 *                was found. 
 *	length  - otherwise
 *
 * side effects:
 *
 ********************************************************/
int trans_tx(int scc_num, void *buf, int length)
{
	int i;
	register IMP_BD *tbd;

	tbd = scc_table[scc_num].tbd;

	/* check the next bd to be used for the transmitter. Be shure not to
	 * erase a bd that is in progress of sending or not confirm yet 
	 */
	if( tbd->status & T_R  || (tbd->length !=0))
		return BUSY;
		
	tbd->length  = (unsigned short)length;
	tbd->buf     = buf;
	tbd->status &= T_W;                     /* Clear all except Wrap bit */
	tbd->status |= T_R | T_L | T_I | T_X; /* Ready to transmittion     */

	/* increment the counter */
	if( tbd->status & T_W )
		scc_table[scc_num].tbd = 
			(IMP_BD *)&imp->pram[scc_num].scc.tbd[0];
	else
		scc_table[scc_num].tbd = (++tbd);

	return length;
}


/********************************************************
 * routine:	trans_rx
 *
 * description:
 *	This function is called by the interrupt routine
 *	upon getting Receive Frame interrupt.
 *	
 *
 * arguments:
 *	scc_num -	the number of the SCC channel (0-2)
 *
 * return code:
 *	None
 *
 ********************************************************/
void trans_rx_buffers(int scc_num)
{
	register IMP_BD *rbd;

	rbd = scc_table[scc_num].rbd;

	while (!(rbd->status & R_E) && (rbd->length !=0 )) {
	/*
	* If Rx error occures - ignore the Rx buffer
	*/
		if (rbd->status & R_ERROR)
			(*scc_table[scc_num].handle_rx_error_ptr)(scc_num, rbd);
		else 
			/* Application may get received buffer */
			(*scc_table[scc_num].handle_rx_buffer_ptr)(scc_num,rbd->buf,rbd->length);
			
		rbd->status &= R_W;
		rbd->buf = (*scc_table[scc_num].alloc_ptr)
			(scc_table[scc_num].max_frame_length);
		if( rbd->buf !=0 ) {
			rbd->length = 1;
			rbd->status |= R_E | R_I | R_X;
		}else
			rbd->length = 0;
		if(rbd->status & R_W)
			rbd = (IMP_BD *)&imp->pram[scc_num].scc.rbd[0];
		else
			rbd++;
	}
	if(rbd->length == 0)
		alloc_mem(scc_num,rbd);

	scc_table[scc_num].rbd = rbd;
}


/********************************************************
 * routine:	trans_tx_done
 *
 * description:
 *	This function is called by the iterrupt routine
 *	upon getting Transmit buffer interrupt.
 *
 * arguments:
 *	scc_num -	the number of the SCC channel (0-2)
 *
 * return code:
 *	None
 *
 ********************************************************/
void trans_tx_done(int scc_num)
{
	register IMP_BD *cbd, *tbd;

	cbd = scc_table[scc_num].confbd;
	tbd = scc_table[scc_num].tbd;

	while (!(cbd->status & T_R)) {
			/* free the buffer */
			(*scc_table[scc_num].free_ptr)(cbd->buf);
			

		cbd->length = 0;        /* signify that this bd is been confirmed */
		if(cbd->status & T_W)
			cbd = (IMP_BD *)&imp->pram[scc_num].scc.tbd[0];
		else
			cbd++;
		if( tbd == cbd)
			break;
	}

	scc_table[scc_num].confbd = cbd;
}

/********************************************************
 * routine:	trans_tx_error
 *
 * description:
 *	This function is called by the iterrupt routine
 *	upon getting Transmit Erorr interrupt.
 *
 * arguments:
 *	scc_num -	the number of the SCC channel (0-2)
 *
 * return code:
 *	None
 *
 ********************************************************/
void trans_tx_error(int scc_num)
{
	register IMP_BD *cbd;

	cbd = scc_table[scc_num].confbd;

	/* first confirm the buffers without errors */
	while (!(cbd->status & T_ERROR)){
			/* free the buffer */
			(*scc_table[scc_num].free_ptr)(cbd->buf);

		if(cbd->status & T_W)
			cbd = (IMP_BD *)&imp->pram[scc_num].scc.tbd[0];
		else
			cbd++;
	}

	scc_table[scc_num].confbd = cbd;

	/* cbd is pointing to the buffer with the error */
	/* notify the user				*/
	(*scc_table[scc_num].handle_tx_error_ptr)(scc_num, cbd);

	/* restart the channel */
	issue_cmd(RESTART_TX | (scc_num << 1)); 
}
/********************************************************
 * routine:	trans_interrupt
 *
 * description:
 *	This function is called by the interrupt routine
 *	upon getting interrupt.
 *
 * arguments:
 *	scc_num -	the number of the SCC channel (0-2)
 *
 * return code:
 *      None
 *
 ********************************************************/
void trans_interrupt(int scc_num)
{
    unsigned short       event;
    struct scc_regs      *regs;

	regs = (struct scc_regs *)&(imp->scc_regs[scc_num]);

	event = regs->event;

	if( event & TRANS_TX  ) 
		trans_tx_done(scc_num);
	if ( event & TRANS_RX )
		trans_rx_buffers(scc_num);
	if( event &  TRANS_BUSY )
		(*scc_table[scc_num].handle_lost_error_ptr)(scc_num);
	if( event &  TRANS_TXE )
		trans_tx_error(scc_num);

	/*
	* cleared handled event bits
	*/
	CLEAR_BIT(regs->event, event);
}
