/****************************************************************/
/*								*/
/*								*/
/* (C) Copyright Motorola Inc, 1994.				*/
/*  All right reserved 						*/
/*                                                              */
/* Author: Uri Trichter                                         */
/*								*/
/* Description:							*/
/*     Simple Driver Set -  protocol independent routines	*/
/*								*/
/* Routines							*/
/*								*/
/*     imp_init	- General IMP initialization			*/
/*     scc_init - SCC channel initialization			*/
/*     scc_interrupt - SCC channel interrupt routine		*/
/*								*/
/*                                                              */
/****************************************************************/

#include "simple.h"
#include "imp.h"
#include "registers.h"

/* define the Globals */
SCC_TABLE scc_table;
IMP     *imp;

/********************************************************
 * routine:	imp_init
 *
 * description:
 *     Initialize the global protocol independent IMP parameters
 *
 * arguments:
 *     imp_start_address - MBAR address of IMP
 *
 * return code:
 *     None
 *
 ********************************************************/

void imp_init( long imp_start_address)
{

	/*
	* Set the address of the imp strat address (MBAR) 
	*/ 
	imp = (IMP *)imp_start_address;
	
	/* set paralell IO */
	imp->pa_control = 0;
	
	/* disable the interrupt controller */
	imp-> intr_mask = 0;
}

/********************************************************
 * routine:	scc_init
 *
 * description:
 *     Initialize the global protocol independent SCC parameters.
 *
 * arguments:
 *     scc_num          - the number of the SCC channel (0-3)
 *
 * return code:
 *     None
 *
 ********************************************************/


/* Exception Vector starts at address 0 (zero ) */
#define VECTOR_LOC(vector)      (4 * (vector))

void scc_init(	int				scc_num, 
		int				max_frame_length,
		alloc_routine			*alloc_buffer,
		free_routine			*free_buffer,
		handle_rx_buffer_routine		*handle_rx_buffer,
		handle_tx_error_routine		*handle_tx_error,
		handle_rx_error_routine		*handle_rx_error,
		handle_lost_error_routine 	*handle_lost_error)
{
	IMP_BD             *bd;
 	int j;
	extern void interrupt scc_isr_0(); /* interrupt service routines */
	extern void interrupt scc_isr_1(); 
	extern void interrupt scc_isr_2(); 

	/* scc_num must be valid and initialized. */
	if (scc_num < 0 || scc_num >= NUM_OF_SCCS )
		return;
    
	/* 
	* INITIALIZE INTERNAL SCC STRUCTURE
	* ---------------------------------
	*/
	scc_table[scc_num].alloc_ptr             = alloc_buffer;
	scc_table[scc_num].max_frame_length      = max_frame_length;
	scc_table[scc_num].free_ptr              = free_buffer;
	scc_table[scc_num].handle_rx_buffer_ptr   = handle_rx_buffer;
	scc_table[scc_num].handle_tx_error_ptr   = handle_tx_error;
	scc_table[scc_num].handle_rx_error_ptr   = handle_rx_error;
	scc_table[scc_num].handle_lost_error_ptr = handle_lost_error;


	/*
	* Initialize Rx BD ring:
	* clear all BD`s status field
	* set R_W on last BD
	* Set the R_X flag for pointing to external memory
	*/
	scc_table[scc_num].rbd = bd = 
				(IMP_BD *)&imp->pram[scc_num].scc.rbd[0];
	for (j = 0; j < NUM_OF_BDS; j++) {
		bd->buf    = (*alloc_buffer)(max_frame_length);
		if( bd->buf != 0){
			bd->length = 1; 
			bd->status = R_E | R_I | R_X; /* set interrupt, ready and external bits*/
		}else{
			bd->length = 0; /* signify allocation problems */
			bd->status = 0;
		}
		bd++;
	}
	(--bd)->status |= R_W; /* set wrap bit */

	/*
	* Initialize Tx BD ring:
	* clear all BD`s status field
	* set T_W on last BD
	*/
	scc_table[scc_num].tbd = bd =  scc_table[scc_num].confbd =
				(IMP_BD *)&imp->pram[scc_num].scc.tbd[0];
	for (j = 0; j <NUM_OF_BDS;j++) {
		bd->status = 0;
		bd->length = 0;
		bd->buf = 0;
		bd++;
	}
	(--bd)->status |= T_W;


	/* prepare the Interrupt Controllers registers */
	imp->intr_mode = 0x00a0; /* locate the interrupts to start at vec 160 */
	j = imp->intr_mode & 0x00e0;

	if (scc_num >= 0 && scc_num < NUM_OF_SCCS) {

		switch( scc_num ) {
			case SCC_0 :
				*((int *)VECTOR_LOC(j | 0xd)) = (int)scc_isr_0;
				/* Set Interrupt Mask reg */
				imp->intr_mask |= INTR_SCC0;
				break;
			case SCC_1 :
				imp->pa_control |= 0x0003; /* rxd2, txd2 */ 
				*((int *)VECTOR_LOC(j | 0xa)) = (int)scc_isr_1;
				/* Set Interrupt Mask reg */
				imp->intr_mask |= INTR_SCC1;
				break;
			case SCC_2:

				imp->pa_control |= 0x0300; /* rxd3, txd3 */ 
				/* Set Interrupt Mask reg */
				imp->intr_mask |= INTR_SCC2;
				*((int *)VECTOR_LOC(j | 0x8)) = (int)scc_isr_2;
				break;
		}
	}
}
    
/********************************************************
 * routine:	scc_interrupt
 *
 * description:
 *     User must call this routine if interrupt on
 *     any of SCC channels was occurred.
 *
 * arguments:
 *     scc_num          - the number of the SCC channel
 *
 * return code:
 *     None
 *
 ********************************************************/
void scc_interrupt(int scc_number)
{
	if (scc_number >= 0 && scc_number < NUM_OF_SCCS ){
		(*scc_table[scc_number].inter_ptr)(scc_number);

		/* clear the interrupt controller in service register */
		switch (scc_number) {
			case SCC_0:
				CLEAR_BIT(imp->intr_inservice, INTR_SCC0);
				break;
			case SCC_1:
				CLEAR_BIT(imp->intr_inservice, INTR_SCC1);
				break;
			case SCC_2:
				CLEAR_BIT(imp->intr_inservice, INTR_SCC2);
				break;
		}
		CLEAR_IPA;
	}
}



/********************************************************
 * routine:	alloc_mem
 *
 * description:
 *     alloc_mem - is a routine that tries to allocate memory 
 *     for replenishing the receive buffers using the user supplied routine. 
 *     The rbd parameter is the first BD that needs to be replenished.
 *     If alloc_mem can not allocate memory it will set the length field
 *     in the bd to zero and will exit, otherwise the length field will be
 *     set to 1.
 *
 * arguments:
 *     scc_num          - the number of the SCC channel
 *     rbd		- pointer to the first receive buffer to be replenished
 *
 * return code:
 *     None
 *
 ********************************************************/
void alloc_mem( int scc_num, IMP_BD *rbd )
{
	register IMP_BD *bd = rbd;
	int i;
	alloc_routine  *alloc_buffer;
	int max_frame_length;

	alloc_buffer = scc_table[scc_num].alloc_ptr;
	max_frame_length = scc_table[scc_num].max_frame_length;

	for( i=0; i < NUM_OF_BDS; i++) {
		if( bd->length == 0) {
			/* try to allocate here */
			bd->buf   = (*alloc_buffer)(max_frame_length);
			if( bd->buf != 0){
				bd->length = 1; /* signify memory allocated */ 
				bd->status = R_E | R_I | R_X; 
			}else
				break;
		}
		
		if(bd->status & R_W)
			bd = (IMP_BD *)&imp->pram[scc_num].scc.rbd[0];
		else
			bd++;
	}
}


/********************************************************
 * routine:	issue_cmd
 *
 * description:
 *     A host command interface routine to CP.
 *
 * arguments:
 *     cmd          - the host command to be issued to the CP
 *		      including the channel number
 *
 * return code:
 *     None
 *
 ********************************************************/
void issue_cmd(unsigned short cmd)
{
	extern IMP	*imp;
	register volatile unsigned char *cr;

	cr = &imp->cr;
	*cr = (cmd | CMD_FLAG);
	while( (*cr) & CMD_FLAG )
		;
}

