/****************************************************************/
/*								*/
/*								*/
/* (C) Copyright Motorola Inc, 1994.				*/
/*  All right reserved 						*/
/*                                                              */
/* Author: Uri Trichter                                         */
/*								*/
/* Description:							*/
/*     Simple Driver Set - UART specific routines.		*/
/*								*/
/* Routines							*/
/*								*/
/*     uart_init						*/
/*     uart_tx							*/
/*     uart_rx_buffers						*/
/*     uart_tx_done						*/
/*     uart_send_brk						*/
/*     uart_send_char_out_of_seq				*/
/*     uart_rx_ccr						*/
/*     uart_interrupt						*/
/*								*/
/****************************************************************/

#include "simple.h"
#include "imp.h"
#include "uart.h"
#include "setup.h"

extern SCC_TABLE scc_table;
extern IMP     *imp;

void uart_interrupt(int scc_num);

/********************************************************
 * routine:	uart_init
 *
 * description:
 *     Initialize the SCC parameter ram and register as a UART
 *     controller. 
 *
 * arguments:
 *     scc_num          - the number of the SCC channel (0-2)
 *     specific_info	- pointer to extra information structure ( comm sett )
 *
 * return code:
 *     None
 *
 ********************************************************/
void uart_init(	int scc_num , void *specific_info)
{
	struct uart_pram *uart_pram;
	struct scc_regs      *regs;
	SETUP		     *setup;
	COMMUNICATION_SETUP  *set;
	CONTROL_CHAR_TBL     *cc_tbl;
	unsigned int div;       /* dividor */
	int i;

	/* scc_num must be valid and initialized. */
	if (scc_num < 0 || scc_num >= NUM_OF_SCCS )
		return;
    
	/* initialize the scc_table struct with the specific protocol interrupt */
	scc_table[scc_num].inter_ptr = uart_interrupt;
	scc_table[scc_num].extra_info = specific_info;
	/*
	* INITIALIZE UART PARAMATER RAM
	* ---------------------------------
	*/

	/* Pointer to corresponding uart parameter ram */
	uart_pram = (struct uart_pram *)&imp->pram[scc_num].scc.pscc.u;

	uart_pram->rfc= 0;		/* Rx FC register */
	uart_pram->tfc= 0;		/* Tx FC register */
	uart_pram->rblen= scc_table[scc_num].max_frame_length;	/* Rx buffer length */
	uart_pram->maxi = 100;		/* maximum idle characters */
	uart_pram->brk = 1;		/* break count register */
	
	uart_pram->rbuf = 0;		/* start from bd = 0 */
	uart_pram->tbuf = (short)(sizeof( IMP_BD)*NUM_OF_BDS);	/* start tbd's from first bd */

	/* clear the counters */
	uart_pram->parity_cnt = 0;	/* Rx parity error counter */
	uart_pram->framing_cnt = 0;	/* Rx framing error counter */
	uart_pram->noise_cnt = 0;	/* Rx noise counter */
	uart_pram->break_cnt = 0;	/* Rx break character counter */

	/* don't use the multidrop addressing schem */
	uart_pram->r_addr1 = 0;	/* address 1 */
	uart_pram->r_addr2 = 0;	/* address 2 */

	setup = (SETUP *)specific_info;

	/* Set pointer to the special_pram which points to control character handler*/
	cc_tbl = (CONTROL_CHAR_TBL *)(&setup->cc_oper);

	for( i=0; i< cc_tbl->num_of_cc_to_handle; i++)
		uart_pram->cc[i] = (cc_tbl->cc_table[i].reject? CC_REJ: 0x0000) |
					cc_tbl->cc_table[i].cc;

	/* signify end of table */
	if( i < 7 )
		uart_pram->cc[i] = CC_INVALID;

	/*
	* INITIALIZE SCC REGISTERS AREA
	* -----------------------------
	*/

	/* Set pointer to the special_pram which points to comm_settings */
	set = (COMMUNICATION_SETUP *)(&setup->comm_sett);

	/* Pointer to SCC registers area */
	regs = (struct scc_regs *)&(imp->scc_regs[scc_num]);

	/* Set scc conf registers to the baud rate*/
	div= (SYSTEM_CLOCK/(set->baud*16)); /* set it to 9600bps */

	if( div > MAX_DIV ){ 
		regs->conf = 0x0001;	/* set div4 bit */ 
		div >>=2;
	}
	else
		regs->conf = 0;
	div--;
	regs->conf |= (div <<1 );	/* SCC configuration register */

	regs->mode = ASYNC_PORT | NORMAL_SW; 

	if(set->parity == ODD_PAR)
                regs->mode |= EN_PARITY | RCV_ODD_PARITY | TX_ODD_PARITY;
        else  if(set->parity == EVEN_PAR)
                regs->mode |= EN_PARITY | RCV_EVEN_PARITY | TX_EVEN_PARITY;
 
        if (set->data_bits == EIGHT_BIT_DATA )
                regs->mode |= EIGHT_BIT_CHAR;
      
	if(set->stop_bits == TWO_STOP_B)
		regs->mode |= TWO_STOP_BITS;

        regs->sync = WHOLE_STOP_BIT;

	/*
	* mask all the events
	*/
	regs->mask =  UART_RX | UART_BUSY | UART_TX; /* SCC mask register */

	if( cc_tbl->num_of_cc_to_handle)
		regs->mask |= UART_CCR;

	/*
	* enable SCC receive transmit operation
	*/
	regs->mode |= ENABLE_TX | ENABLE_RX;
}


/********************************************************
 * routine:	uart_tx
 *
 * description:
 *	Write the details of a given buffer to a transmit
 *	buffer descriptor of given uart channels.
 *
 * arguments:
 *	scc_num -	the number of the SCC channel (0-2)
 *	buf	-	pointer to the buffer to be transmitted
 *	length	-	buffer length
 *
 * return code:
 *	BUSY	- Indicate that no empty transmit buffer descriptor
 *                was found. 
 *	length  - otherwise
 *
 * side effects:
 *
 ********************************************************/
int uart_tx(int scc_num, void *buf, int length)
{
	int i;
	register IMP_BD *tbd;

	tbd = scc_table[scc_num].tbd;

	/* check the next bd to be used for the transmitter. Be shure not to
	 * erase a bd that is in progress of sending or not confirm yet 
	 */
	if( tbd->status & T_R  || (tbd->length !=0) )
		return BUSY;
		
	tbd->length  = (unsigned short)length;
	tbd->buf     = buf;
	tbd->status &= T_W;                     /* Clear all except Wrap bit */
	tbd->status |= T_R | T_I | T_X; /* Ready to transmittion     */

	/* increment the counter */
	if( tbd->status & T_W )
		scc_table[scc_num].tbd = 
			(IMP_BD *)&imp->pram[scc_num].scc.tbd[0];
	else
		scc_table[scc_num].tbd = (++tbd);

	return length;
}


/********************************************************
 * routine:	uart_rx
 *
 * description:
 *	This function is called by the interrupt routine
 *	upon getting Receive Frame interrupt.
 *	
 *
 * arguments:
 *	scc_num -	the number of the SCC channel (0-2)
 *
 * return code:
 *	None
 *
 ********************************************************/
void uart_rx_buffers(int scc_num)
{
	register IMP_BD *rbd;
	short needs_allocation = 0;

	rbd = scc_table[scc_num].rbd;

	while (!(rbd->status & R_E) && (rbd->length !=0 )) {
	/*
	* If Rx error occures - ignore the Rx buffer
	*/
		if (rbd->status & R_ERROR)
			(*scc_table[scc_num].handle_rx_error_ptr)(scc_num, rbd);
		else 
			/* Application may get received buffer */
			(*scc_table[scc_num].handle_rx_buffer_ptr)(scc_num,rbd->buf,rbd->length);
			
		rbd->status &= R_W;
		rbd->buf = (*scc_table[scc_num].alloc_ptr)
			(scc_table[scc_num].max_frame_length);
		if( rbd->buf !=0 ) {
			rbd->length = 1;
			rbd->status |= R_E | R_I | R_X;
		}else {
			rbd->length = 0;
			needs_allocation = 1;
		}
		if(rbd->status & R_W)
			rbd = (IMP_BD *)&imp->pram[scc_num].scc.rbd[0];
		else
			rbd++;
	}
	if(needs_allocation)
		alloc_mem(scc_num,rbd);

	scc_table[scc_num].rbd = rbd;
}


/********************************************************
 * routine:	uart_tx_done
 *
 * description:
 *	This function is called by the iterrupt routine
 *	upon getting Transmit buffer interrupt.
 *
 * arguments:
 *	scc_num -	the number of the SCC channel (0-2)
 *
 * return code:
 *	None
 *
 ********************************************************/
void uart_tx_done(int scc_num)
{
	register IMP_BD *cbd, *tbd;

	cbd = scc_table[scc_num].confbd;
	tbd = scc_table[scc_num].tbd;

	while (!(cbd->status & T_R)) {
			/* free the buffer */
			(*scc_table[scc_num].free_ptr)(cbd->buf);
			

		cbd->length = 0;	/* signify that this bd is been confirmed */

		if(cbd->status & T_W)
			cbd = (IMP_BD *)&imp->pram[scc_num].scc.tbd[0];
		else
			cbd++;
		if( tbd == cbd)
			break;
	}

	scc_table[scc_num].confbd = cbd;
}

/********************************************************
 * routine:	uart_send_break
 *
 * description:
 *	Send a brk character. The brk length is decided from the brk 
 * 	parameter ram register.
 *
 * arguments:
 *	scc_num -	the number of the SCC channel (0-2)
 *
 * return code:
 *      None
 *
 ********************************************************/
void uart_send_brk(int scc_num)
{
	issue_cmd(STOP_TX | (scc_num <<1));
	issue_cmd(RESTART_TX | (scc_num << 1));
}


/********************************************************
 * routine:	uart_send_char_out_of_seq
 *
 * description:
 *	Send a charater out of sequance order
 *
 * arguments:
 *	scc_num -	the number of the SCC channel (0-2)
 *	ch	-	character to be send out of order.
 *
 * return code:
 *      None
 *
 ********************************************************/
void uart_send_char_out_of_seq(int scc_num, unsigned char ch)
{
	struct uart_pram *uart_pram;
	uart_pram = (struct uart_pram *)&imp->pram[scc_num].scc.pscc.u;

	uart_pram->cc[8] = (CC_INVALID | REA | ch);	/* not using address, cts and interrupt */
	
	/* use a polling ( not interrupt) on finish tx the character */
	while( uart_pram->cc[8] & REA )
			;	
}


/********************************************************
 * routine:	uart_rx_ccr
 *
 * description:
 *	Interrupt on a control character occured 
 *
 * arguments:
 *	scc_num -	the number of the SCC channel (0-2)
 *
 * return code:
 *      None
 *
 ********************************************************/
void uart_rx_ccr(int scc_num )
{
	unsigned char ch;
	struct uart_pram *uart_pram;
	CONTROL_CHAR_TBL *tbl;
	int i;

	uart_pram = (struct uart_pram *)&imp->pram[scc_num].scc.pscc.u;

	/* read the rccr reg to get the ccr */
	ch = (uart_pram->ccr & CC_CHAR);

	tbl = (CONTROL_CHAR_TBL *)&((SETUP *)scc_table[scc_num].extra_info)->cc_oper;
	/* look in cc table for this cc */
	for( i=0; i< tbl->num_of_cc_to_handle; i++)
		if( ch == tbl->cc_table[i].cc) {
			(*tbl->cc_table[i].cc_func_oper)();
			break;
		}
}
			
	
/********************************************************
 * routine:	uart_interrupt
 *
 * description:
 *	This function is called by the interrupt routine
 *	upon getting interrupt.
 *
 * arguments:
 *	scc_num -	the number of the SCC channel (0-2)
 *
 * return code:
 *      None
 *
 ********************************************************/
void uart_interrupt(int scc_num)
{
    unsigned short       event;
    struct scc_regs      *regs;

	regs = (struct scc_regs *)&(imp->scc_regs[scc_num]);

	event = regs->event;

	if( event &  UART_CCR )
		uart_rx_ccr(scc_num);
	if( event & UART_TX ) 
		uart_tx_done(scc_num);
	if ( event & UART_RX )
		uart_rx_buffers(scc_num);
	if( event &  UART_BUSY )
		(*scc_table[scc_num].handle_lost_error_ptr)(scc_num);

	/*
	* cleared handled event bits
	*/
	CLEAR_BIT(regs->event, event);
}


